@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: protocol.h,v 1.12 2010/01/06 17:01:38 murch Exp $
  */
 
 #ifndef _INCLUDED_PROTOCOL_H
@@ -47,7 +45,16 @@
 
 #include "saslclient.h"
 
-#define MAX_CAPA 8
+enum {
+    /* protocol types */
+    TYPE_STD           = 0,    /* protocol fits standard template */
+    TYPE_SPEC          = 1     /* non-standard needs special functions */
+};
+
+struct stdprot_t;
+struct backend;
+
+#define MAX_CAPA 9
 
 enum {
     /* generic capabilities */
@@ -61,8 +68,6 @@ enum {
     */
 };
 
-struct protocol_t;
-
 struct banner_t {
     u_char auto_capa;		/* capability response sent automatically
 				   in banner? */
@@ -74,13 +79,20 @@ struct capa_t {
     unsigned long flag;
 };
 
+#define CAPAF_MANY_PER_LINE	(0)
+#define CAPAF_ONE_PER_LINE	(1<<0)
+#define CAPAF_SKIP_FIRST_WORD	(1<<1)
+#define CAPAF_QUOTE_WORDS	(1<<2)
+#define CAPAF_DASH_STUFFING	(1<<3)
+
 struct capa_cmd_t {
     const char *cmd;		/* [OPTIONAL] capability command string */
     const char *arg;		/* [OPTIONAL] capability command argument */
     const char *resp;		/* end of capability response */
-    char *(*parse_mechlist)(const char *str, struct protocol_t *prot);
-				/* [OPTIONAL] parse capability string,
-				   returns space-separated list of mechs */
+    void (*postcapability)(struct backend *);
+				/* [OPTIONAL] called after capabilities,
+				 * received from server, are parsed */
+    int formatflags;		/* CAPAF* constants above */
     struct capa_t capa[MAX_CAPA+1];/* list of capabilities to parse for
 				      (MUST end with NULL entry) */
 };
@@ -99,9 +111,7 @@ struct simple_cmd_t {
     const char *ok;		/* success response */
 };
 
-struct protocol_t {
-    const char *service;	/* INET service name */
-    const char *sasl_service;	/* SASL service name */
+struct stdprot_t {
     struct banner_t banner;
     struct capa_cmd_t capa_cmd;
     struct tls_cmd_t tls_cmd;
@@ -111,4 +121,23 @@ struct protocol_t {
     struct simple_cmd_t logout_cmd;
 };
 
+struct protocol_t {
+    const char *service;       /* INET service name */
+    const char *sasl_service;  /* SASL service name */
+    unsigned type;
+    union {
+       struct stdprot_t std;
+       struct {
+	   /* special protocol */
+	   int (*login)(struct backend *s, const char *userid,
+			sasl_callback_t *cb, const char **status,
+			int noauth);
+	   int (*ping)(struct backend *s, const char *userid);
+	   int (*logout)(struct backend *s);
+       } spec;
+    } u;
+};
+
+
+
 #endif /* _INCLUDED_PROTOCOL_H */
